#ifndef LCD_H_
#define LCD_H_	1

#include <avr/pgmspace.h>

#define SPI_REG			DDRB
#define SPI_PORT		PORTB

#define LCD_CS			PB0
#define LCD_RST			PB1
#define LCD_CMD			PB2
#define LCD_BL			PB3
#define LCD_NC			PB4
#define LCD_MOSI		PB5
#define LCD_SCK			PB7

void LCD_init(void);
void LCD_end(void);

void LCD_set_range(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);
void LCD_set_pixel(uint16_t color);

void LCD_draw_pixel(int16_t x, int16_t y, uint16_t color);
void LCD_fill_screen(uint16_t color);

void LCD_invert_display(uint8_t inv);

uint16_t LCD_get_color(uint8_t r, uint8_t g, uint8_t b);

#define TFT141_TFT_WIDTH	128
#define TFT141_TFT_HEIGHT	128

#define TFT141_NOP		0x00
#define TFT141_SWRESET		0x01
#define TFT141_RDDID		0x04
#define TFT141_RDDST		0x09

#define TFT141_SLPIN		0x10
#define TFT141_SLPOUT		0x11
#define TFT141_PTLON		0x12
#define TFT141_NORON		0x13

#define TFT141_INVOFF		0x20
#define TFT141_INVON		0x21
#define TFT141_GSET			0x26
#define TFT141_DISPOFF		0x28
#define TFT141_DISPON		0x29
#define TFT141_CASET		0x2A
#define TFT141_RASET		0x2B
#define TFT141_RAMWR		0x2C
#define TFT141_RAMRD		0x2E

#define TFT141_PTLAR		0x30
#define TFT141_VERTSCRL		0x33
#define TFT141_COLMOD		0x3A
#define TFT141_MADCTL		0x36

#define TFT141_FRMCTR1		0xB1
#define TFT141_FRMCTR2		0xB2
#define TFT141_FRMCTR3		0xB3
#define TFT141_INVCTR		0xB4
#define TFT141_DISSET5		0xB6

#define TFT141_PWCTR1		0xC0
#define TFT141_PWCTR2		0xC1
#define TFT141_PWCTR3		0xC2
#define TFT141_PWCTR4		0xC3
#define TFT141_PWCTR5		0xC4
#define TFT141_VMCTR1		0xC5
#define TFT141_VMCTR2		0xC6
#define TFT141_VMCTROFF		0xC7

#define TFT141_RDID1		0xDA
#define TFT141_RDID2		0xDB
#define TFT141_RDID3		0xDC
#define TFT141_RDID4		0xDD

#define TFT141_PWCTR6		0xFC

#define TFT141_DELAY		0x80

#define TFT141_GMCTRP1		0xE0
#define TFT141_GMCTRN1		0xE1
#define TFT141_GAMRSEL		0xF2

#define TFT141_BRIGHTNESS	0x0004
#define TFT141_BLACK		0x0000
#define TFT141_RED		0x001F
#define TFT141_BLUE		0xF800
#define TFT141_GREEN		0x07E0
#define TFT141_YELLOW		0x07FF
#define TFT141_MAGENTA		0xF81F
#define TFT141_CYAN		0xFFE0
#define TFT141_WHITE		0xFFFF

#endif	/* LCD_H_ */
