#define __PROG_TYPES_COMPAT__

#define F_CPU	16000000

#include <avr/pgmspace.h>
#include <avr/io.h>

#include <util/delay.h>
#include <stdio.h>

#include "lcd.h"
#include "gfx.h"




#include <stdlib.h>
#include <string.h>

#define SENSORS_NO      	2
#define MOISTURE_SENSOR 	1
#define TEMPERATURE_SENSOR	0


#include "lcd.h"
int i = 0;

/* Initializez contextul (senzorul) */
void ADC_init(void) {

	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

/* Citesc date de la senzor. */
int ADC_get(uint8_t channel) {

	ADMUX = (ADMUX & ~(0x1F << MUX0)) | channel;
	
	ADCSRA |= (1 << ADSC);
	
	while (ADCSRA & (1 << ADSC));
	
	return ADC;
}

/* Calculez in procente valoarea obtinuta de senzor pentru
   obtinerea unui afisaj user friendly. */
float percent_moisture(int mo)
{
	float percent;
	int d1, d2, one_point, rap_points, new_points;

	if (mo > 300) {
		// 10 - 20%
		d1 = 800 - 300;
		d2 = 20 - 10;
		one_point = d1 * 1 / d2;
		rap_points = mo - 300;
		new_points = rap_points / one_point;
		percent = 10 + (d2 - new_points);
		return percent;
	}
	if (mo > 150) {
		// 20 - 35%
		d1 = 300 - 150;
		d2 = 35 - 20;
		one_point = d1 * 1 / d2;
		rap_points = mo - 150;
		new_points = rap_points / one_point;
		percent = 20 + (d2 - new_points);
		return percent;
	}
	if (mo > 100) {
		// 35 - 50%
		d1 = 150 - 100;
		d2 = 50 - 35;
		one_point = d1 * 1 / d2;
		rap_points = mo - 100;
		new_points = rap_points / one_point;
		percent = 35 + (d2 - new_points);
		return percent;
	}
	if (mo > 25) {
		// 50 - 75%
		d1 = 100 - 25;
		d2 = 75 - 50;
		one_point = d1 * 1 / d2;
		rap_points = mo - 25;
		new_points = rap_points / one_point;
		percent = 50 + (d2 - new_points);
		return percent;
	}
	if (mo > 0) {
		// 75 - 100%
		d1 = 25 - 0;
		d2 = 100 - 75;
		one_point = d1 * 1 / d2;
		rap_points = mo - 0;
		new_points = rap_points / one_point;
		percent = 75 + (d2 - new_points);
		return percent;
	}
	if (mo <= 0) {
		return 99;
	}
}

/* Afisez un mesaj de bun venit. */
static void draw_welcome(int16_t x, int16_t y)
{
	GFX_set_text_size(2);
	GFX_set_rotation(0);

	GFX_set_text_color(TFT141_GREEN);
	GFX_set_cursor(x, y);
	GFX_print_str("Welcome");
}

/* Afisez valorile obtinute de la senzor. */
static void draw_measurments(int16_t x, int16_t y)
{
	char line2[16];
	int i, j, sensors[SENSORS_NO];

	for (i = 0; i < SENSORS_NO; i++) {
		sensors[i] = ADC_get(i);
	}

	GFX_set_text_size(2);
	GFX_set_rotation(0);

	GFX_set_text_color(TFT141_RED);
	GFX_set_cursor(x, y);
	GFX_print_str("Umid. sol:");

	y =  y + 24;
	GFX_set_cursor(x, y);
	itoa(sensors[MOISTURE_SENSOR], line2, 10);
	GFX_print_str(line2);

	y =  y + 24;
	GFX_set_cursor(x, y);
	GFX_print_str("T. ambient");

	y =  y + 24;
	GFX_set_cursor(x, y);
	GFX_print_str("23 grade");

	/* Fac refresh doar liniei pe care afisez valorile. */
	float res;
	y = y - 48;
	while (1) {
		GFX_set_cursor(x, y);
		for (i = 0; i < 150; ++i) {
			for (j = 0; j < 16; ++j) {
				LCD_draw_pixel(i + x, j + y, TFT141_WHITE);
			}
		}

		for (i = 0; i < SENSORS_NO; i++) {
			sensors[i] = ADC_get(i);
		}
		
		res = percent_moisture(sensors[MOISTURE_SENSOR]);
		itoa(res, line2, 10);
		strcat(line2, "%   ");
		// itoa(sensors[MOISTURE_SENSOR], line2, 10);
		GFX_print_str(line2);
		itoa(sensors[MOISTURE_SENSOR], line2, 10);
		GFX_print_str(line2);

		_delay_ms(2000);
	}
}


static void draw_goodbye(int16_t x, int16_t y)
{
	GFX_set_text_size(2);
	GFX_set_rotation(0);

	GFX_set_text_color(TFT141_RED);
	GFX_set_cursor(x, y);
	GFX_print_str("Goodbye!");
}

int main(void)
{
	LCD_init();
	GFX_init(128, 160);

	LCD_fill_screen(TFT141_BLACK);
	draw_welcome(10, 64);
	_delay_ms(2000);

	ADC_init();

	LCD_fill_screen(TFT141_WHITE);
	draw_measurments(5, 40);
	_delay_ms(2000);

	LCD_fill_screen(TFT141_WHITE);
	draw_goodbye(5, 144);
	_delay_ms(2000);

	LCD_fill_screen(TFT141_BLUE);
	GFX_draw_circle(20, 100, 10, TFT141_WHITE, 1);
	_delay_ms(2000);

	LCD_end();

	return 0;
}
